require 'acres_client'
require 'talk_consumer'
require 'net_shell/net_shell'
require 'event_manager.rb'
require 'object_builder/object_builder'
require "a4tools/version"

def symbolify(item)
  if item.is_a? Array
    return item.map! { |value| symbolify value }
  elsif not item.is_a? Hash
    return item
  end

  new_hash = {}
  item.each { |key, value| new_hash[key.to_sym] = symbolify value }
  new_hash
end

def talk
  $talk ||= A4Tools::TalkConsumer.new(talk_file)
end

def refresh_talk
  $talk = A4Tools::TalkConsumer.new(talk_file)
end

def talk_file
  return $localtalk if not $localtalk.nil? and File.exists?($localtalk)
  A4Tools::TalkConsumer.new(File.join(File.dirname(__FILE__), "talk.json"))
end

def events
  $events ||= A4Tools::EventManager.new
end

def try_connection(hostname, port)
  s = TCPSocket.new(hostname, port) rescue false
  s != false
end

# array of { :host, :port }
def try_connection_options(options, timeout=2)
  mutex = Mutex.new
  condition = ConditionVariable.new
  result = nil

  mutex.synchronize do
    options.each do |opt|
      Thread.new do
        if try_connection(opt[:host], opt[:port]) then
          result = opt
          mutex.synchronize { condition.signal }
        end
      end
    end

    condition.wait(mutex, timeout)
  end
  result
end
