category "Protocol manipulation"
description "Access an individual field in a JSON object"
usage "field_path"

help "Finds a field path inside a JSON object supplied over standard input. For example, use 'tok.rand' to find the 'rand' field inside the 'tok' object inside the object."

validate { args.length <= 2 }

tab :type do
  talk.definition[:class].map { |defn| talk.truncated_name(defn[:name]) }
end

tab :class do
  talk.definition[:class].map { |defn| talk.truncated_name(defn[:name]) }
end

opt :class, "Excerpt the first field of the given class name", :type => :string
opt :no_hint, "Do not force result object to contain an __class field"
opt :type, "Assume the object has the specified type", :type => :string
opt :no_bypass, "Do not automatically bypass JSONRPCRequest, JSONRPCResponse and NamedObjectWrapper toplevels"

run do
  object, type = toplevel_value
  return show_error("Invalid JSON") if object.nil?

  data, type = talk.locate_data(object, args[1], params[:type])
  return show_error("Unable to locate field in JSON.") if data.nil?

  result = if data.is_a? Hash or data.is_a? Array then
    unless params[:class].nil?
      data = find_class(params[:class], data, type)
      return show_error("Unable to locate class type in JSON") if data.nil?
      type = params[:class]
    end
    data[:__class] = talk.expand_name(type) if data.is_a? Hash and not params[:no_hint]
    JSON.generate(data)
  else
    data
  end
end

###

def find_class(target, data, type=nil)
  return find_class_in_array(target, data, type) if data.is_a? Array

  type ||= talk.guess_class(data)
  cls = talk.class_named(type) rescue nil
  
  if cls.nil? then
    data.each do |key, value|
      searchval = find_class(target, value)
      return searchval unless searchval.nil?
    end
  else
    cls[:field].each do |field|
      value = data[field[:name].to_sym]
      fieldtype = talk.expand_name(field[:type].first)
      next if value.nil? or fieldtype.nil?
      return value if talk.name_matches?(fieldtype, target)
    end
  end

  nil
end

def find_class_in_array(target, data, type)
  data.each do |datum|
    result = find_class(target, datum, type)
    return result unless result.nil?
  end

  return nil
end

def toplevel_value
  json = @input.read_all
  object = symbolify(JSON.parse(json)) rescue nil
  type = (talk.expand_name(params[:type]) rescue nil) || talk.guess_class(object) rescue nil
  
  return nil if type.nil? or object.nil?
  return [object, type] if params[:no_bypass]
  return bypass(object, type)
end

def bypass(object, type)
  case type
  when "com.acres4.common.info.NamedObjectWrapper"
    return bypass(object[:body], object[:className])
  when "com.acres4.common.info.JSONRPCRequest"
    return bypass(object[:params], talk.guess_class(object[:params]))
  when "com.acres4.common.info.JSONRPCResponse"
    return bypass(object[:result], talk.guess_class(object[:result]))
  else
    return [object, type]
  end
end
