category "Shell management"
description "Show help for a command"

tab 0 do
  @shell.built_ins
end

help <<-EOS
Show help for a specific built-in command, or outline all built-in commands if none is available.
EOS

run { args.length == 1 ? list_all_commands : list_command(args[1]) }

###

def list_all_commands
  (categories.map { |cat| list_category(cat) }).join("\n\n")
end

def list_command(name)
  return show_error("#{name}: command not found") unless @shell.has_built_in?(name)
  cmd = @shell.built_in(name).new(@shell)
  cmd.educate
end

def categories
  (@shell.built_ins.map { |name| @shell.built_in(name).category_text }).sort.uniq
end

def list_category(category)
  commands = @shell.built_ins.select { |name| @shell.built_in(name).category_text == category }
  cat = category.style(:category)
  cmds = (commands.sort.map do |name|
    cmd = @shell.built_in(name).new(@shell)
    usermark = cmd.user_defined? ? "@".style(:userdefined) : " "
    "#{sprintf("%25s", name.to_s).style(:command)}#{usermark} -- #{cmd.description}"
  end).join("\n")

  [cat, cmds].join("\n")
end
