category "Protocol manipulation"
description "Inject a JSON string from the command line into another JSON object from STDIN"
usage "field_path json"

validate { args.length == 3 }

run do
  path = args[1].split(".")
  last = path.pop
  json = @input.read_all

  parent = nil
  toplevel = object = symbolify(JSON.parse(json)) rescue nil
  injected = JSON.parse(args[2]) rescue args[2]

  return show_error("Invalid source object") if injected.nil?
  return show_error("Invalid replacement object") if injected.nil?

  until path.empty? do
    field_name = path.first
    path = path[1..-1]
    object = field_value(object, field_name)
    return show_error("Path not in JSON object") if object.nil?
  end
  
  return json if object.nil?
  object[last.to_sym] = injected
  toplevel.to_json
end

###

def field_value(object, field_name)
  m = field_name.to_s.match(/^([a-zA-Z0-9_]+)\[([0-9]+)\]$/)
  if m.nil? then
    return object[field_name.to_sym] rescue nil
  else
    name = m[1]
    value = object[name.to_sym][m[2].to_i] rescue nil
    return value
  end
end
