category "Kai"
description "Generate machine events"
usage "[event_code machine_location]"

tab do
  events.events.map { |event| event[:key] }
end

tab :machine do
  machine_list
end

opt :amount, "Cash value associated with event. Used for jackpot and handpays.", :type => :integer
opt :tier, "Player tier level name", :type => :string
opt :first_name, "Player first name", :type => :string
opt :last_name, "Player last name", :type => :string
opt :employee, "Employee card ID", :type => :integer
opt :patron, "Patron card I", :type => :integer
opt :machine, "Machine location", :type => :string

opt :allow_duplicate, "Allow duplicate machines when spamming with random machine IDs"
opt :spam, "Spam the event repeatedly", :type => :integer, :default => 1
opt :spam_interval, "Specify number of seconds between spams in milliseconds", :type => :integer, :default => 1000

opt :list_names, "List Kai events, alphabetized by name"

run do
  if args.length == 1 then
    list = events.events
    (list.sort! { |a,b| a[:key] <=> b[:key] }) if params[:list_names]
    (events.events.map do |event|
      code = sprintf("%5d", event[:code]).style(:event_code)
      key = sprintf("%35s", event[:key].to_s).style(:event_key)

      "#{code}  #{key} -- #{event[:description]}"
    end).join("\n")
  else
    machine = args[2] rescue random_machine
    code = event_code(args[1])
    return show_error("Unknown event code #{code}") if code.nil?
    return show_error("Unable to connect QA test interface") unless qa.connect_if_needed
    return show_error("Only have #{machine_list.length} machines to choose from; either reduce spam count or allow duplicates") if params[:spam] > machine_list.length and not params[:allow_duplicate]

    index = 0
    allowed_list = machine_list
    params[:spam].times do
      index = index + 1
      machine = params[:machine] || allowed_list.sample
      allowed_list.delete(machine) unless params[:allow_duplicate]

      message = build_request(args[1..-2].sample, machine)
      puts "Sending event #{index} of #{params[:spam]}: #{event_description(message)}"
      send_event(message)
      sleep params[:spam_interval] * 1.0/1000.0 unless index == params[:spam]
    end

    ""
  end
end

##

def event_description(event)
  name = event_name(event[:eventList][0][:event][:idEventCode]) || event[:eventList][0][:event][:idEventCode]
  location = event[:eventList][0][:location]
  "#{name} @ #{location}"
end

def machine_list
  @shell.client(:config).machines rescue []
end

def random_machine
  machine_list.sample
end

def send_event(message)  
  qa.inject_token(message)
  jsonrpc = qa.jsonrpc_message(:qaTestEvent, "qatest", nil, message)
  qa.send_message(jsonrpc).body
end

def build_request(code, machine)
  {
    __class: "com.acres4.common.info.mercury.qatest.QATestEventRequest",
    tok:nil,
    eventList:[ qa_event_table_item(code, machine) ]
  }
end

def qa_event_table_item(code, machine)
  {
    event:event_table_item(code),
    location:machine
  }
end

def event_table_item(code)
  {
    idEventCode:code,
    hostEvent:host_event(code),
    collectStats:false,
    receiptTime:Time.now.to_i,
    playerRating:player_rating
  }
end

def host_event(code)
  {
    priKey:0,
    hostDeviceId:nil,
    eventCode:code.to_s,
    idPatron:params[:patron] || 0,
    idEmployee:params[:employee] || 0,
    amount:params[:amount] || 0,
    meters:nil,
    eventTime:Time.now.to_i,
    hostTime:Time.now.to_i
  }
end

def player_rating
  return nil unless params[:patron] or params[:firstName] or params[:lastName] or params[:loyaltyTierStr]

  {
    idPatron:params[:patron] || 0,
    firstName:params[:first_name] || "Little",
    lastName:params[:last_name] || "Timmy",
    loyaltyTierStr:params[:tier] || "Wood"
  }
end

def event_by_identifier(identifier)
  identifier ||= args[1]
  identifier = identifier.to_i if identifier.is_a? String and identifier.match(/^[0-9]+$/)
  events.events.each do |event|
    return event if(event[:key].to_s == identifier.to_s or event[:code] == identifier)
  end

  pp identifier
  pp args[1]
  nil
end

def event_code(identifier=nil)
  event_by_identifier(identifier)[:code]
end

def event_name(identifier=nil)
  event_by_identifier(identifier)[:key]
end
