category "Talk"
description "Find references to a Talk class"
usage "class_name"
help "Lists all classes and protocols referencing a particular Talk class."

validate { args.length == 2 }

tab 0 do
  talk.things_named_like(args[1], [:class]).map { |thing| talk.truncated_name thing[:name] }
end

run do
  search_cls = talk.class_named(args[1])
  return show_error("#{args[1]}: no such class in talk") if search_cls.nil?
  full_name = search_cls[:name]

  methods = list_methods(full_name)
  fields = list_fields(full_name)

  if methods.length > 0 then
    puts "Methods (#{methods.length})"
    methods.each { |method| show_method(method[:protocol], method[:method]) }
  end

  if fields.length > 0 then
    puts "Fields (#{fields.length})"
    fields.each { |field| show_field(field[:class], field[:field]) }
  end

  ""
end

###

def list_methods(name)
  methods = []
  talk.definition[:protocol].each do |proto|
    proto[:method].each do |method|
      methods.push({protocol:proto, method:method}) if(talk.name_matches?(name, method[:request]) or talk.name_matches?(name, method[:response]))
    end
  end

  methods
end

def list_fields(name)
  fields = []
  talk.definition[:class].each do |cls|
    cls[:field].each do |field|
      fields.push({class:cls, field:field}) if talk.name_matches?(name, field[:type].first)
    end
  end

  fields
end

def show_method(protocol, method)
  puts "\t#{protocol[:name].style(:protcol_name)}: #{method[:name].style(:method_name)} #{method[:request].style(:class)} -> #{method[:response].style(:class)}"
end

def show_field(cls, field)
  puts "\t#{cls[:name].style(:class)}: #{field[:name].style(:field_name)}#{field[:type][1..-1].join("")}"
end
