category "Usher"
description "List everything about a site"
usage "siteId1 siteId2 ..."

opt :no_deploy, "Do not show deployment information"

tab do
  sites = (usherm[:usherSystemId][:usherSites].map { |site| site[:idSite] } +
    usherm[:usherSystemId][:usherSites].map { |site| site[:name] }) rescue []
  sites.reject { |site| args.include? site.to_s }
end

run do
  usherm.ensure_fresh
  deployment.ensure_fresh
  if args.length >= 2 then
    args[1..-1].each do |arg|
      site_id = site_id_for_arg(arg)
      if site_id.nil? then
        puts "Unknown site '#{site_id}'"
        next
      end

      show_site(site_id)
      puts
    end
  else
    show_all_sites
  end

  ""
end

###

def site_id_for_arg(arg)
  return arg.to_i if arg.is_a? Fixnum or arg.match(/^[0-9]+$/)
  usherm[:usherSystemId][:usherSites].each { |site| return site[:idSite] if site[:name] == arg  }
  nil
end

def show_all_sites
  sites = (usherm[:usherSystemId][:usherSites].map { |site| site[:idSite] }).sort
  sites.each do |site_id|
    site = usherm.site_with_id(site_id)
    site_id_str = sprintf("%6d", site_id).style(:id)
    puts "##{site_id_str}: #{site[:name]} (#{site[:siteDescription]})"
  end
end

def show_site(site_id)
  site = usherm.site_with_id(site_id)
  show_usher_site(site_id, site)
  show_host_info(site_id)
  show_publications(deployment.query(:buildPublicationList,site_id)) unless params[:no_deploy]
end

def yes_no(b)
  b ? "yes" : "no"
end

def timestamp(ts)
  DateTime.strptime(ts.to_s,'%s').strftime("%Y-%m-%d %H:%M:%S").style(:timestamp)
end

def expiration(mod)
  mod[:permanent] ? "Never" : timestamp(mod[:expiration])
end

def show_usher_site(site_id, site)
  if site.nil?
    puts "Site #{site_id.to_s.style(:id)}: Unknown (not in usher)"
    return
  end

  puts "Site".style(:heading) +" #{site[:idSite].to_s.style(:id)}: #{site[:name]} (#{site[:siteDescription]})"
  puts "\t         Timezone: #{site[:zoneinfoId].style(:timezone)}"
  mod = site[:moduleProvision]
  if mod then
    mask = sprintf("%16x", mod[:modules]).style(:mask)
    puts "\t          Modules: #{mask}"
    puts "\t     Max machines: #{mod[:maxAllowedMachines]}"
    puts "\t           Active: #{yes_no(mod[:active])}"
    puts "\tPermanent license: #{yes_no(mod[:permanent])}"
    puts "\t       Expiration: #{expiration(mod)}"
    puts "\t Renewal attempts: #{mod[:renewalAttemtps]}"
  end
  site[:siteRange].each do |range|
    puts "\t             CIDR: #{range[:cidrBlock].style(:cidr)} #{range[:siteRangeDescription]}"
  end
end

def show_host_info(site_id)
  puts "\tHosts".style(:heading)
  usherm.providers_for_site(site_id).each do |provider|
    host = usherm.host_with_id(provider[:idSystemHost])
    product = sprintf("%20s", provider[:product]).style(:product)
    puts "\t\t#{product}: #{host[:hostName].style(:host)}"
  end
end

def show_publications(publications)
  puts "\tDeployments".style(:heading)
  publications[:orders].keys.sort.each do |app|
    next if params[:app] and not talk.name_matches?(app, params[:app])
    list_site_app(app.to_s, publications[:orders][app])
  end
end

def list_site_app(app, orders)
  pub = orders.first
  return if params[:jeb] and pub[:buildInfo][:buildId] != params[:jeb]
  return if params[:commit] and pub[:buildInfo][:commitId] != params[:commit]
  return if params[:branch] and not(pub[:buildInfo][:branches].include? params[:branch])
  puts "\t\t#{timestamp(pub[:publicationTime])} #{app.style(:class)} -- #{build_info(pub[:buildInfo])}"
end

def build_info(build)
  name = build[:buildId].style(:build_name)
  branches = build[:branches].join(",").style(:branch)
  "#{name} [#{branches}]"
end
