require 'socket'
require 'thread'

category "System administration"
description "Open an SSH connection to the specified host."
usage "host [user]"

help "Opens an SSH session to the specified host. No command line arguments are passed."

validate do
  return "must connect usherm first" if usherm.nil?
  args.length == 2
end

tab 0 do
  (usherm[:usherSystemId][:systemHosts].map { |host| host_supports_ssh?(host) ? host[:hostName] : nil }).compact rescue []
end

run do
  params = parse_params(args)
  usherm.ensure_fresh
  acl = usherm.acl_for_service(params[:host], "ssh")
  return show_error("#{params[:host]}: usher does not list ssh ACL for host") unless acl
  try_acl(params, acl) ? "" : "#{args[0]}: #{params[:host]}: unable to connect to host on any address"
end

###

def try_acl(params, acl)
  host = usherm.host_named(params[:host])
  service = usherm.service_for_id(acl[:idService])
  options = [
    { host:host[:ipAddrInt], port:service[:port] },
    { host:host[:ipAddrExt], port:acl[:port] }
  ]

  result = try_connection_options(options)
  unless result.nil? then
    process = fork { exec "ssh -l #{params[:user]} -p #{result[:port]} #{result[:ip]}" }
    Process.waitpid(process)
  end

  not result.nil?
end

def host_supports_ssh?(host)
  not usherm.acl_for_service(host, "ssh").nil?
end

def parse_params(args)
  if args.length == 3
    return { host:args[1], user:args[2] }
  else
    if args[1].include? "@" then
      comps = args[1].split("@")
      return { host:comps[1], user:comps[0] }
    else
      return { host:args[1], user:"acres4" }
    end
  end
end

