require 'talk'

category "Talk"
description "Select a given version of Talk"
usage "[branch_name]"
help <<-EOS
Selects a given version of Talk for use in netshell.
EOS

opt :repo, "Use the indicated local repository instead of default"
opt :commit, "Select a Talk version in terms of a git commit hash instead of a branch", :type => :string
opt :update, "Ensure that the acres4/talk repository is up-to-date"
opt :jeb, "Select a Talk version in terms of a jeb instead of a branch", :type => :string
opt :talkversion, "Use a specific version of maketalk", :type => :string

tab :commit do
  return list_commits.map { |commit| commit[:hash] }
end

tab :jeb do
  return list_commits.map { |commit| commit[:jeb] }
end

tab 0 do
  return list_branches
end

validate do
  return true if args.length == 2 and !params[:jeb] and !params[:commit]
  return true if args.length == 1 and (params[:jeb] or params[:commit])
end

run do
  clone_repo unless repo_exists?
  fetch if params[:update]
  target = params[:jeb] || params[:commit] || args[1]
  checkout(target)
  make_talk
  ""
end

###

def repo
  params[:repo] || @shell.shelldir("talk")
end

def talk_json_path
  $localtalk
end

def repo_exists?
  File.exist?(File.join(repo, ".git"))
end

def fetch
  `cd #{repo} && git fetch`
end

def checkout(id)
  id = Jebediah.new.process(id)[:result] if params[:jeb]
  `cd #{repo} && git checkout origin/#{id}`
end

def make_talk
  version = params[:talkversion] ? " _#{params[:talkversion]}_" : ""
  `cd #{repo} && maketalk#{version} --upgrade --json info > #{talk_json_path}`
  refresh_talk
end

def clone_repo
  puts "Cloning respository into #{repo}"
  `cd #{File.join(repo, "..")} && git clone git@github.com:acres4/talk`
end

def list_commits
  return [] unless repo_exists?
  jeb = Jebediah.new
  `cd #{repo} && git log --pretty=format:"%an:%h:%s"`.split("\n").map do |commit|
    components = commit.split(":")
    { author:components[0], hash:components[1], jeb:jeb.process(components[1])[:result].join(" "), notes:components[2] }
  end
end

def list_branches
  return [] unless repo_exists?
  branches = `cd #{repo} && git branch -a --no-color`.split("\n").map { |x| x.gsub(/^\*? */,"").gsub(/^.*-> /,"").gsub(/^(remotes\/)?origin\//,"") }
  branches &= branches # remove dupes
  branches.sort
end
