category "Talk"
description "Find talk entities matching a constant value or string"
usage "value"

help <<-EOS
Attempts to find a string or number in a Talk glossary or enumeration. All matching constants are displayed. Constants are case-sensitive.

Example:
Show all constants equal to 10
  #{command} 10

Show all terms equal to Mercury
  #{command} Mercury

EOS

validate { args.length == 2 }

tab 0 do
  (talk.definition[:glossary].inject([]) { |terms, glossary| terms + glossary[:term] }).map { |t| t[:value] }
end 

run { translate(args[1]) }

###

def translate(value)
  is_numeric = (not(value.is_a?(String)) or value.match(/^[-+]?[0-9]+(.[0-9]+)?$/) != nil)

  candidates = case is_numeric
  when true
    talk.definition[:enumeration].inject([]) do |c, enum|
      c + (enum[:constant].select { |constant| constant[:value] == value.to_f }).map { |x| { collection:enum, constant:x } }
    end
  when false
    talk.definition[:glossary].inject([]) do |c, glossary|
      c + (glossary[:term].select { |term| term[:value] == value }).map { |x| { collection:glossary, constant:x } }
    end
  end

  return "No matches for #{value}" if candidates.empty?

  (candidates.map { |item| "#{item[:constant][:name].style(:constant_name)} (#{talk.truncated_name item[:collection][:name].style(:collection_name)}): #{item[:constant][:description]}" }).join("\n")
end

