category "Usher"
description "Determine usher information for current IP and serial number"
usage "[serial_number]"

help <<-EOS
Prints usher information for clients requesting from the specified IP address and serial number.

If no IP or serial number are supplied, #{command.style(:command)} uses the host's actual IP address and the "#{"serial".style(:environment)}" environment variable.
EOS

opt :full, "Instruct usher to give all possible hosts instead of trying to guess which host is appropriate. (e.g. use \"#{"usherAlt".style(:method_name)}\" method)"
opt :json, "Print result as JSON"
opt :url, "Use usher mirror at specified URL", :type => :string
opt :ip, "Use usher as if coming from IP address", :type => :string

tab 0 do
  (usherm[:devSiteInfoSync][:devSiteInfoArray].map { |info| info[:serialNumber] }) rescue []
end

run do
  params[:serial] = args[1] if args.length > 0
  result = A4Tools::UsherClient.new(params[:url]).list_servers_raw(params)
  return params[:json] ? result.body : render_result(result)
end

###

def render_result(response)
  return ("Unable to contact usher") if response.nil?
  return "Usher returned status #{response.code}" if response.code.to_i < 200 or response.code.to_i >= 300
  begin
    payload = JSON.parse(response.body)
    return ("Usher returned non-JSON RPC response") unless payload["jsonrpc"] == "2.0"
    return ("Usher returned error: \"#{payload["error"]}\"") unless payload["error"].nil?
    return ("Usher did not include result") if payload["result"].nil?
    return ("Usher server returned non-usher response") unless payload["result"]["className"] == "com.acres4.common.info.usher.UsherResponse"
    return ("Usher response did not include body") if payload["result"]["body"].nil?
    return render_success(payload["result"]["body"])
  rescue JSON::ParserError
    return ("Usher returned non-JSON response")
  end
end

def render_success(result)
  s  = "       Site ID: #{result["siteID"]}\n"
  s += "     Site name: #{result["siteName"]}\n"
  s += "Remote address: #{result["remoteAddr"]}\n\n"

  result["hosts"].each do |host|
    s += "Product: #{host["product"]} (#{host["host"]}:#{host["port"]})\n\t#{host["url"]}, #{host["ctx"]}, System #{host["systemId"]}\n\n"
  end

  s
end

