category "Usher"
description "Manage development devices in Usher"
usage [
  "[siteId]                   -- list development devices in usher",
  "serial siteId [comment]    -- update development device with specified serial number"
]

help <<-EOS
Updates the development device with the specified serial in UsherM to use the specified site ID. If a comment supplied, the device's comment field is updated to the supplied comment.

If no such serial is registered, a new registration is created.

Must have previously connected to UsherM using #{"usherm_connect".style(:environment)}.
EOS

validate do
  return show_error("must connect usherm first") if usherm.nil?
  args.length <= 4
end

tab 0 do
  usherm[:devSiteInfoSync][:devSiteInfoArray].map { |info| info[:serialNumber] } rescue []
end

run do
  if args.length == 1  or args.length == 2 then
    usherm.ensure_fresh
    devices = usherm[:devSiteInfoSync][:devSiteInfoArray]
    devices = devices.reject { |info| info[:idSite].to_i != args[1].to_i } if args.length == 2
    (devices.map do |info|
      "#{sprintf("%3d", info[:idDevSiteInfo]).to_s.light_blue}  #{info[:serialNumber].red} -> Site ##{info[:idSite].to_s.bold}, #{info[:description]}"
    end).join("\n")
  else
    description = args[3] rescue nil
    usherm.update_device(args[1], description, args[2].to_i, "Added via netshell by #{`whoami`.chomp}@#{`hostname`.chomp}")
    ""
  end
end

