require 'trollop'

module A4Tools
  class Command
    attr_accessor :args, :input, :output, :error, :shell
    attr_reader :status

    def initialize(args, input=nil, output=nil, error=nil, shell=nil)
      @input = input || StandardInput.new
      @output = output || StandardOutput.new
      @error = error || StandardOutput.new
      @args = args
      @shell = shell
    end

    def is_built_in?
      built_in_methods.include? args[0].to_sym
    end

    def run_built_in
      built_in = @shell.built_in(args[0]).new(@shell)
      built_in.execute(@input, @output, @error, args)

      @status = built_in.status
    end

    def built_in_methods
      @shell.built_ins
    end

    def tab_complete_built_in
      case args.length
      when 0..1
        built_in_methods
      else
        @shell.tab_complete_built_in(args)
      end
    end

    def get_option_set
      if args.length <= 1 or is_built_in?
        tab_complete_built_in.map { |opt| opt.to_s || "" }
      else
        []
      end
    end

    def tab_complete
      fragment = args.last || ""
      get_option_set.select { |opt| opt.to_s.start_with? fragment }
    end

    def make_commandline(args)
      (args.map do |arg|
        if arg.match(/\s/).nil? then
          arg
        else
          arg.shellescape
        end
      end).join(" ")
    end

    def run_external
      begin
        cmdline = make_commandline(args)
        Open3.popen3(cmdline) do |stdin, stdout, stderr, wait_thr|
          begin
            pid = wait_thr[:pid]

            stdin.write(@input.read)
            stdin.close

            @status = wait_thr.value

            @output.write(stdout.read)
            stdout.close
            stderr.close
          rescue Errno::EPIPE
            @error.write("-netshell: #{args[0]}: #{stderr.read}\n")
            @status = 127
          end
        end
      rescue Errno::ENOENT
        @output.write("-netshell: #{args[0]}: command not found\n")
        @status = 127
      end

      @status
    end

    def run
      return "" if args.empty? or args[0].nil?
      return run_built_in if is_built_in?
      return run_external
    end
  end
end
