require 'open3'

class StringBuffer
  attr_accessor :string
  def initialize
    @string = ""
  end

end

module A4Tools
  class StandardOutput
    attr_accessor :color

    def write(s)
      s = s.unstyle unless @color
      $stdout.write s.gsub("\t", "  ").gsub("\n","\r\n")
    end

    def close
    end

    def print(s="")
      write s.to_s
    end

    def puts(s="")
      write s.to_s+"\n"
    end
  end

  class PipeBuffer
    attr_accessor :color
    attr_reader :buffer

    def initialize
      @buffer = ""
    end

    def read
      @color ? @buffer : @buffer.unstyle
    end

    def read_all
      @color ? @buffer : @buffer.unstyle
    end

    def write(s)
      @buffer += s
    end

    def puts(s)
      if s.is_a? Array then
        s.each { |e| puts e.to_s }
      else
        write(s.to_s+"\n")
      end
    end

    def printf(s, *a)
      write(sprintf(s.to_s, *a))
    end

    def print(s="")
      write(s.to_s)
    end

    def close
    end
  end

  class FileOverwriteOutput < StandardOutput
    def initialize(filename)
      @filename = filename
    end

    def access_type
      "w"
    end

    def write(s)
      s = s.unstyle unless @color
      @fd ||= File.open(@filename, access_type)
      @fd.write(s)
      @fd.flush
    end

    def close
      @fd.close
    end
  end

  class FileAppendOutput < FileOverwriteOutput
    def access_type
      "a"
    end
  end

  class StandardInput
    def read
      begin
        STDIN.read_nonblock(256)
      rescue IO::WaitReadable
        ""
      rescue EOFError
        nil
      end
    end

    def read_all
      buffer = ""
      while (line = read) != nil
        buffer += line
      end
      buffer.chomp
    end
  end

  class FileReadInput
    def initialize(filename)
      @filename = filename
    end

    def read
      IO.read(@filename)
    end

    def read_all
      IO.read(@filename)
    end
  end
end
