require 'jebediah'

description "Populated from git and version info"
generate do
  version = get_version.split(".")
  commits = get_commits

  {
    __class:"com.acres4.common.info.AppInfo",
    appName:"netshell",
    product:"Mercury",

    major: version[0].to_i,
    minor: version[1].to_i,
    revision: version[2].to_i,
    sourceRevision: commits.first[:sha1],
    buildId: commits.first[:jeb],
    supportingRevisions: (commits.map do |commit|
      {
        repository: commit[:repository],
        revision: commit[:sha1],
        alias: commit[:jeb],
        timestamp: commit[:timestamp],
        author: commit[:author],
        commitNotes: commit[:notes],
        branches: commit[:branches]
      }
    end),

    sha1sum:nil,
    currentTalkVersion:"mike",
  }
end

###

def repo_info(directory, repository=nil)
  begin
    info = {}
    info[:repository] = repository || File.basename(directory)
    info[:sha1] = `cd "#{directory}"; git log --format="%H" -n1`.chomp
    info[:jeb] = Jebediah.new.process(info[:sha1])[:result].join(" ")
    info[:timestamp] = `cd "#{directory}"; git log --format="%at" -n1`.chomp.to_i
    info[:author] = `cd "#{directory}"; git log --format="%an" -n1`.chomp
    info[:notes] = `cd "#{directory}"; git log --format="%s" -n1`.chomp

    info[:branches] = (`cd "#{directory}"; git branch --contains #{info[:sha1]}`.split("\n").map do |line|
      line.length > 0 ? line.gsub(/^[\s*]*/, "").gsub(/\s*$/, "") : nil
    end).compact
    return info
  rescue
    return nil
  end
end

def fake_repo_info
  {
    repository: "a4tools",
    sha1:"67021ff1f82c941454a6d2bffa7a2ffde0a6320a",
    jeb:"exquisitely eased cougar",
    timestamp:1399271669,
    author:"Jonas Acres",
    notes:"Progress commit",
    branches:["master"]
  }
end

def get_a4tools
  repo_info(File.dirname(__FILE__), "a4tools")
end

def get_talk
  repo_info(File.join(File.dirname(__FILE__), "../../../../talk"), "talk")
end

def get_commits
  #list = [ get_a4tools, get_talk ].compact
  list = [ fake_repo_info ] if list.nil? or list.empty?
end

def get_version
  VERSION
end
